const {
    default: TestCaseHookDefinition,
} = require('@cucumber/cucumber/lib/models/test_case_hook_definition');
const jq = require('node-jq');
const { env, drgWithAttach, environment, environmentFromService } = require('./util');

// DDS Data Access calls DDS Services in DRG and returns specified results

// To get PDC entries to services are called
async function getPDCEntries(attach, PatientCPR, version) {
    // First calling DDS Registry IDWS
    id = 'sfsk.iti18'
    const body = {
        id: id,
        role: 'sds.foces',
        endpoint: environment(environmentFromService.get(id)),
        fields: {
            cprNumber: PatientCPR,
        },
        'auth-attributes': {
            cpr: PatientCPR,
        },
    };

    // Saving response
    let sfskRegistry = await drgWithAttach(attach, body);

    // Finding entries/documents in response
    let documentlist =
        sfskRegistry.body.response.main.Envelope.Body.AdhocQueryResponse
            .RegistryObjectList.ExtrinsicObject;

    let res;
    if (version == '2') {
        // Finding PDC entry/document in document list
        res = await jq.run(
            `.[]|select(.Classification[]|select(.__nodeRepresentation== "urn:ad:dk:medcom:pdc-v2.0:full"))`,
            documentlist,
            { input: 'json', output: 'json' }
        );
    } else if (version == '3') {
        res = await jq.run(
            `. | select(.Classification[] | .__nodeRepresentation == "urn:ad:dk:medcom:pdc-v3.0:full")`,
            documentlist,
            { input: 'json', output: 'json' }
        );
    } else {
        res = await jq.run(
            `.[]|select(.Classification[]|select(.__nodeRepresentation== "PDC"))`,
            documentlist,
            { input: 'json', output: 'json' }
        );
    }
    // Saving home community, document unique and repository id from document
    let documentUniqueId = res.__id;
    let repositoryUniqueId = res.__id.substring(
        0,
        documentUniqueId.indexOf('^')
    );

    // Body to calling DGWS
    id = 'sfsk.iti43Medarbejder'
    const bodyDdsRepository = {
        id: id,
        role: 'test.annebonde',
        endpoint: environment(environmentFromService.get(id)),
        fields: {
            CitizenCivilRegistrationNumber: PatientCPR,
            OnBehalfOf: false,
            ConsentOverride: false,
            repositoryUniqueId: repositoryUniqueId,
            documentUniqueId: documentUniqueId,
        },
    };

    let sfskRepository = await drgWithAttach(attach, bodyDdsRepository);

    let entry_ddsRepositry;

    console.log(sfskRepository.body.response.attachments[0])

    if (sfskRepository.body.response.attachments[0] != null) {
        entry_ddsRepositry = sfskRepository.body.response.attachments[0].ClinicalDocument.component
            .structuredBody.component.section.entry;
    } else {
        entry_ddsRepositry = sfskRepository.body.response.main.Envelope.Body.RetrieveDocumentSetResponse.
            DocumentResponse.Document._data.ClinicalDocument.component.structuredBody.component.section.entry;
    }

    return entry_ddsRepositry;
}

async function getDDSRegistryWithMoces(attach, Role, PatientCPR) {
    id = 'ddsregistry.dgws'
    const body = {
        id: id,
        role: Role,
        endpoint: environment(environmentFromService.get(id)),
        fields: {
            cprNumber: PatientCPR,
            CitizenCivilRegistrationNumber: PatientCPR,
        },
        'auth-attributes': {
            cpr: PatientCPR,
        },
    };

    return await drgWithAttach(attach, body);
}

async function getDDSRepository(attach, patientCPR, documentUniqueId) {
    id = 'ddsrepository.dgws'
    const bodyDdsRepository = {
        id: id,
        role: 'test.annebonde',
        endpoint: environment(environmentFromService.get(id)),
        fields: {
            CitizenCivilRegistrationNumber: patientCPR,
            homeCommunityId: '1.2.208.176.8.1.12',
            repositoryUniqueId: '1.2.208.176.43210.8.10.11',
            documentUniqueId: documentUniqueId,
        },
    };

    return await drgWithAttach(attach, bodyDdsRepository);

}

async function getDDSRegistryWithPoces(attach, PatientCPR) {
    id = 'ddsregistry.idws'
    const body = {
        id: id,
        role: 'test.citizen',
        endpoint: environment(environmentFromService.get(id)),
        fields: {
            cprNumber: PatientCPR,
        },
        'auth-attributes': {
            cpr: PatientCPR,
        },
    };
    return await drgWithAttach(attach, body);
}

module.exports = {
    getPDCEntries,
    getDDSRegistryWithMoces,
    getDDSRegistryWithPoces,
    getDDSRepository,
};
