const { env, drgWithAttach, environment, environmentFromService } = require('./util');

async function CreateOrganDonorRegistration(attach, PatientCPR) {
    id = 'odr.CreateOrganDonorRegistration'
    const body = {
        id: id,
        role: 'test.citizen',
        endpoint: environment(environmentFromService.get(id)),
        fields: {
            CPR: PatientCPR,
            permissionType: 'LIMITED_WITH_RESEARCH',
            permissionForHeart: 'true',
            permissionForLungs: 'true',
            permissionForLiver: 'true',
            permissionForPancreas: 'true',
            permissionForKidneys: 'true',
            permissionForCornea: 'true',
            permissionForSmallIntestine: 'true',
            permissionForSkin: 'true',
            requiresRelativeAcceptance: 'true',
        },
        'auth-attributes': {
            cpr: PatientCPR,
        },
    };
    const response = await drgWithAttach(attach, body);

    return response;
}

async function GetOrganDonorRegistration(attach, PatientCPR) {
    id = 'odr.GetOrganDonorRegistration'
    const body = {
        id: id,
        role: 'test.citizen',
        endpoint: environment(environmentFromService.get(id)),
        fields: {
            CPR: PatientCPR,
        },
        'auth-attributes': {
            cpr: PatientCPR,
        },
    };
    let organDonorRegistrationResponse = await drgWithAttach(attach, body);

    return organDonorRegistrationResponse.body.response.main.Envelope.Body
        .GetOrganDonorRegistrationResponse.ClinicalDocument.component
        .structuredBody.component.section.entry.observation.value;
}

async function HasOrganDonorRegistration(attach, PatientCPR) {
    id = 'odr.HasOrganDonorRegistration'
    const body = {
        id: id,
        role: 'test.citizen',
        endpoint: environment(environmentFromService.get(id)),
        fields: {
            CPR: PatientCPR,
        },
        'auth-attributes': {
            cpr: PatientCPR,
        },
    };
    let hasOrganDonorRegistrationResponse = await drgWithAttach(attach, body);

    return hasOrganDonorRegistrationResponse.body.response.main.Envelope.Body
        .HasOrganDonorRegistrationResponse.registrationExists._text;
}

async function DeleteOrganDonorRegistration(attach, PatientCPR) {
    id = 'odr.DeleteOrganDonorRegistration'
    const body = {
        id: id,
        role: 'test.citizen',
        endpoint: environment(environmentFromService.get(id)),
        fields: {
            CPR: PatientCPR,
        },
        'auth-attributes': {
            cpr: PatientCPR,
        },
    };
    response = await drgWithAttach(attach, body);

    return response;
}

async function GetOrganDonorRegistrationAdmin(attach, PatientCPR) {
    id = 'odrAdmin.GetOrganDonorRegistration'
    const body = {
        id: id,
        role: 'test.annebonde',
        endpoint: environment(environmentFromService.get(id)),
        fields: {
            CPR: PatientCPR,
        },
        'auth-attributes': {
            cpr: PatientCPR,
        },
    };
    let GetorganDonorRegistrationResponse = await drgWithAttach(attach, body);

    return GetorganDonorRegistrationResponse.body.response.main.Envelope.Body
        .GetOrganDonorRegistrationResponse.ClinicalDocument.component
        .structuredBody.component.section.entry.observation.value;
}

async function CreateOrganDonorRegistrationAdmin(attach, PatientCPR) {
    id = 'odrAdmin.CreateOrganDonorRegistration'
    const body = {
        id: id,
        role: 'test.annebonde',
        endpoint: environment(environmentFromService.get(id)),
        fields: {
            CPR: PatientCPR,
            permissionType: 'LIMITED_WITH_RESEARCH',
            permissionForHeart: 'true',
            permissionForLungs: 'true',
            permissionForLiver: 'true',
            permissionForPancreas: 'true',
            permissionForKidneys: 'true',
            permissionForCornea: 'true',
            permissionForSmallIntestine: 'true',
            permissionForSkin: 'true',
            requiresRelativeAcceptance: 'true',
        },
        'auth-attributes': {},
    };
    await drgWithAttach(attach, body);
}

async function UpdateOrganDonorRegistrationAdmin(attach, PatientCPR) {
    id = 'odrAdmin.UpdateOrganDonorRegistration'
    const body = {
        id: id,
        role: 'test.annebonde',
        endpoint: environment(environmentFromService.get(id)),
        fields: {
            CPR: PatientCPR,
            permissionType: 'LIMITED',
            permissionForHeart: 'false',
            permissionForLungs: 'false',
            permissionForLiver: 'true',
            permissionForPancreas: 'false',
            permissionForKidneys: 'true',
            permissionForCornea: 'false',
            permissionForSmallIntestine: 'false',
            permissionForSkin: 'false',
            requiresRelativeAcceptance: 'false',
        },
        'auth-attributes': {},
    };
    await drgWithAttach(attach, body);
}

async function UpdateOrganDonorRegistration(attach, PatientCPR) {
    id = 'odr.UpdateOrganDonorRegistration'
    const body = {
        id: id,
        role: 'test.citizen',
        endpoint: environment(environmentFromService.get(id)),
        fields: {
            CPR: PatientCPR,
            permissionType: 'LIMITED',
            permissionForHeart: 'false',
            permissionForLungs: 'false',
            permissionForLiver: 'true',
            permissionForPancreas: 'false',
            permissionForKidneys: 'true',
            permissionForCornea: 'false',
            permissionForSmallIntestine: 'false',
            permissionForSkin: 'false',
            requiresRelativeAcceptance: 'false',
        },
        'auth-attributes': {
            cpr: PatientCPR,
        },
    };
    await drgWithAttach(attach, body);
}

async function DeleteOrganDonorRegistrationAdmin(attach, PatientCPR) {
    id = 'odrAdmin.DeleteOrganDonorRegistration'
    const body = {
        id: id,
        role: 'test.annebonde',
        endpoint: environment(environmentFromService.get(id)),
        fields: {
            CPR: PatientCPR,
        },
        'auth-attributes': {},
    };
    response = await drgWithAttach(attach, body);

    return response;
}

module.exports = {
    CreateOrganDonorRegistration,
    GetOrganDonorRegistration,
    HasOrganDonorRegistration,
    DeleteOrganDonorRegistration,
    GetOrganDonorRegistrationAdmin,
    CreateOrganDonorRegistrationAdmin,
    DeleteOrganDonorRegistrationAdmin,
    UpdateOrganDonorRegistrationAdmin,
    UpdateOrganDonorRegistration,
};
