const v4 = require('uuid').v4;
const {
    Before,
    BeforeAll,
    After,
    Given,
    Then,
    When,
} = require('@cucumber/cucumber');
const jq = require('node-jq');
const { expect, parseRelativeDate, oid } = require('./util');
let {
    drgWithAttach,
    patient_cpr,
    number_of_relatives,
    environment,
    serviceFromReadable,
    environmentFromService,
    setEnvironment,
    resetEnvironment
} = require('dataaccess/util');

const roles = new Map(require('../config/roles.json'));
const chosenRoles = new Map(require('../config/chosenroles.json'));
const fs = require('fs/promises');
const crypto = require("crypto")


const newBody = (id, role, endpoint) => ({
    id,
    role,
    endpoint: endpoint,
    fields: {},
    'auth-attributes': {},
});

const fgvhrBody = (id, role, endpoint) => ({
    id,
    role,
    endpoint: endpoint,
    fields: {},
    'auth-attributes': {},
    'chosen-role': {},
});

Given('datasæt {string} med felterne', function (name, dataTable) {
    const data = this.parseDataTable(dataTable);
    // If statement tilføjet for at fjerne z i data, for Fibs Newborn registration ikke kan håndtere at der er z i slutningen.
    if (name === 'FIBS Newborn hent') {
        data.BirthTime = data.BirthTime.slice(0, -1)

    }
    this.datasets.set(name, data);
    console.log(name, data);
});

Given('{string} er {string}', function (variable, value) {
    if (value.match(/nytuniktid/i)) {
        value = v4();
    } else if (value.match(/now/i)) {
        value = parseRelativeDate(value);
    } else if (value.match(/nytuniktoid/i)) {
        value = oid();
    } else if (value.match(/patient_cpr/i)) {
        value = patient_cpr;
    } else if (value.match(/number_of_relatives/i)) {
        value = number_of_relatives;
    }

    this.variables.set(variable, value);
    console.log(variable, value);
});
//anvendes til IDSAS hent salt, da denne ikke har nogle parametre man kan sende med i DRG kaldet. 
When('{string} kaldes gennem DRG', async function (service) {
    console.log(service);
    const id = serviceFromReadable.get(service)
    if (id == null) {
        throw new Error(`Unknown DRG service id "${service}"`);
    }
    defaultEnv = environmentFromService.get(id)

    if (id == null) {
        throw new Error(`Unknown DRG service id "${service}"`);
    }

    env = environment(defaultEnv)

    const body = newBody(id, this.role, env);
    //body.fields = { NoFieldsTextBox: "true" }
    console.log(body);
    const res = await drgWithAttach(this.attach, body);
    this.responses.push(res);

});

When(
    '{string} kaldes gennem DRG med datasæt {string}',
    { timeout: -1 },
    async function (service, name) {
        console.log(service, name);
        const id = serviceFromReadable.get(service)
        defaultEnv = environmentFromService.get(id)

        if (id == null) {
            throw new Error(`Unknown DRG service id "${service}"`);
        }

        env = environment(defaultEnv)

        let body = newBody(id, this.role, env);
        const dataset = this.datasets.get(name);
        body.fields = dataset;

        //Gældende for FGVHR administrationskald, skal der tilføjes en chosenrole
        //fixturen '{string} har en nationalrolle {string}' definere chosenRole.
        if (id.substring(0, 3) == 'fgv' || this.chosenRole != undefined && this.chosenRole.title == 'SundAssistR1') {
            const fgvhrBody = (id, role, endpoint) => ({
                id,
                role,
                endpoint: endpoint || env,
                fields: {},
                'auth-attributes': {},
                'chosen-role': {},
            });
            let fgvhrbody = fgvhrBody(id, this.role, env)
            fgvhrbody.fields = dataset;
            if (this.chosenRole != undefined) {
                fgvhrbody['chosen-role'] = this.chosenRole;
            }
            body = fgvhrbody
        }

        //Rep er specielt i det man skal angive et CPR i auth selvom man ikke har et i body. Vi hardkoder til vi har en løsning på det
        if (dataset.cprNumber) {
            body['auth-attributes'].cpr = dataset.cprNumber;
        }
        if (dataset.CitizenCivilRegistrationNumber) {
            body['auth-attributes'].cpr =
                dataset.CitizenCivilRegistrationNumber;
        }
        if (dataset.PatientCpr) {
            body['auth-attributes'].cpr = dataset.PatientCpr;
        }
        if (dataset.patientId) {
            body['auth-attributes'].cpr = dataset.patientId;
        }
        if (dataset.CPR) {
            body['auth-attributes'].cpr = dataset.CPR;
        }
        if (dataset.identifier) {
            body['auth-attributes'].cpr = dataset.identifier;
        }
        if (dataset.PersonCivilRegistrationIdentifier) {
            body['auth-attributes'].cpr = dataset.PersonCivilRegistrationIdentifier;
        }

        if (dataset.serviceStartTime) {
            dataset.serviceStartTime = parseRelativeDate(dataset.serviceStartTime).replaceAll(/[^0-9 ]/g, "");
            console.log(dataset.serviceStartTime)
        }
        if (dataset.serviceStopTime) {
            dataset.serviceStopTime = parseRelativeDate(dataset.serviceStopTime).replaceAll(/[^0-9 ]/g, "");
            console.log(dataset.serviceStopTime)
        }

        console.log(body);
        console.log(body.fields);
        const res = await drgWithAttach(this.attach, body);
        this.responses.push(res);
    }
);

When(
    '{string} kalder {string} gennem DRG med datasæt {string} og svar gemmes som {string}',
    { timeout: -1 },
    async function (role, service, datasetName, responseName) {
        console.log(service, datasetName);
        const id = serviceFromReadable.get(service)
        defaultEnv = environmentFromService.get(id)

        if (id == null) {
            throw new Error(`Unknown DRG service id "${service}"`);
        }

        env = environment(defaultEnv)

        role = this.replaceVariables(role);
        const roleId = roles.get(role);
        if (roleId == null) {
            throw new Error(`Unknown role "${role}"`);
        }

        let body = newBody(id, roleId, env);
        const dataset = this.datasets.get(datasetName);
        body.fields = dataset;

        //Herunder sættes auth-attributes
        //Rep er specielt i det man skal angive et CPR i auth selvom man ikke har et i body. Vi hardkoder til vi har en løsning på det
        if (dataset.cprNumber) {
            body['auth-attributes'].cpr = dataset.cprNumber;
        }
        if (dataset.CitizenCivilRegistrationNumber) {
            body['auth-attributes'].cpr =
                dataset.CitizenCivilRegistrationNumber;
        }
        if (dataset.PatientCpr) {
            body['auth-attributes'].cpr = dataset.PatientCpr;
        }
        if (dataset.CPR) {
            body['auth-attributes'].cpr = dataset.CPR;
        }
        if (dataset.identifier) {
            body['auth-attributes'].cpr = dataset.identifier;
        }
        if (dataset.cprfuldmagt) {
            body['auth-attributes']['cpr-fuldmagt'] = dataset.cprfuldmagt;
        }

        if (this.chosenRole != undefined) {
            body['chosen-role'] = this.chosenRole
        }

        const res = await drgWithAttach(this.attach, body);
        this.variables.set(responseName, res);
        this.responses.push(res);
    }
);

Given(
    'environment er {string}',
    function (environment) {
        //Sætter jenkinsEnv i util til at være = setEnvironment
        setEnvironment(environment)
    }
);

Given(
    'environment er {string}',
    function (environment) {
        //Sætter jenkinsEnv i util til at være = setEnvironment
        setEnvironment(environment)
    }
);

Then('modtager service svarer {string}', function (status) {
    switch (status) {
        case 'ok':
            if (this.getLastResponse().body.code != 200) {
                console.log("Faultstring: \n" + JSON.stringify(
                    this.getLastResponse().body.response.main.Envelope.Body.Fault.faultstring, null, 2)
                );
            }
            expect(this.getLastResponse().body.code).to.eql(200);
            break;
        case 'fejl':
            if (this.getLastResponse().body.code != 200) {
                console.log("Faultstring: \n" + JSON.stringify(
                    this.getLastResponse().body.response.main.Envelope.Body.Fault.faultstring, null, 2)
                );
            }
            expect(this.getLastResponse().body.code).to.eql(500);

            break;
        default:
            expect(this.getLastResponse().body.code).to.eql(status);
    }
});

Given('bruger er en {string}', function (role) {
    const roleId = roles.get(role);
    if (!roleId) {
        throw new Error(`Unknown role "${role}"`);
    }
    this.role = roleId;

    resetEnvironment()
});

Then('retursvar fejltekst er {string}', function (errormsg) {
    // Denne fixture validerer om en given fejltekst i service svarer til den man forventer
    console.log(errormsg);
    expect(
        this.getLastResponse().body.response.main.Envelope.Body.Fault
            .faultstring._text
    ).to.eql(errormsg);
});

Then('retursvar registry-fejltekst er {string}',
    async function (errormsg) {
        let response = this.getLastResponse().body.response.main.Envelope.Body
        let errors;
        if (response.AdhocQueryResponse != undefined) {
            errors = this.getLastResponse().body.response.main.Envelope.Body.AdhocQueryResponse
                .RegistryErrorList.RegistryError;
        } else {
            errors = this.getLastResponse().body.response.main.Envelope.Body
                .RegistryResponse.RegistryErrorList.RegistryError;
        }
        if (Array.isArray(errors)) {
            error = await jq.run(
                `.[]|select(.__codeContext=="${errormsg}").__codeContext`,
                errors,
                { input: 'json', output: 'json' }
            );
        } else {
            error = errors.__codeContext;
        }
        expect(error).to.eql(errormsg);
    });

Then(
    '{string} + {int} er lig {string}',
    function (oldNumber, number, newNumber) {
        // Funktionen sammenligner 2 tal inkl evt + eller - et antal eks tal1 + 1 er lig tal2 (som er sand hvis tal1=1 og tal2=2)
        oldNumber = this.replaceVariables(oldNumber);
        newNumber = this.replaceVariables(newNumber);
        console.log(oldNumber, newNumber);
        expect(oldNumber + number).to.eql(newNumber);
    }
);

Then('{string} er lig {string}', function (oldNumber, newNumber) {
    oldNumber = this.replaceVariables(oldNumber);
    newNumber = this.replaceVariables(newNumber);
    expect(oldNumber).to.eql(newNumber);
});

Then('certifikat {string} er lig certifikat {string}', function (inputCertificate, expectedCertificate) {
    inputCertificate = this.replaceVariables(inputCertificate);
    expectedCertificate = this.replaceVariables(expectedCertificate);

    inputBuf = Buffer.from(inputCertificate, 'base64')
    expectedBuf = Buffer.from(expectedCertificate, 'base64')

    requestCertificate = new crypto.X509Certificate(inputBuf)
    testCertificate = new crypto.X509Certificate(expectedBuf)

    console.log("Requestets certifikat:")
    console.log(requestCertificate)
    console.log("Forventet certifikat:")
    console.log(testCertificate)

    expect(requestCertificate.subject).to.equal(testCertificate.subject)
    expect(requestCertificate.issuer).to.equal(testCertificate.issuer)
    expect(requestCertificate.validFrom).to.equal(testCertificate.validFrom)
    expect(requestCertificate.validTo).to.equal(testCertificate.validTo)

    expect(requestCertificate.serialNumber).to.equal(testCertificate.serialNumber)
    expect(requestCertificate.fingerprint).to.equal(testCertificate.fingerprint)
});

Given('{string} er valgt', function (Element) {
    Element = this.replaceVariables(Element);
    expect(Element).not.to.eql(null);
});

Given('teknisk datasæt {string} med felterne', function (name, dataTable) {
    const data = this.parseDataTable(dataTable);

    this.datasets.set(name, data);
});

Given('{string} har nationalrolle {string}', function (role, chosenRole) {

    let roleid = roles.get(role);
    if (roleid == null) {
        throw new Error(`Unknown role "${role}"`);
    }
    let chosenRoleid = chosenRoles.get(chosenRole);
    if (chosenRoleid == null) {
        throw new Error(`Unknown chosen-role "${chosenRole}"`);
    }
    this.role = roleid;
    this.chosenRole = chosenRoleid
});

Then('requestCertifikat gemmes som {string}', async function (certificate) {
    const response = this.getLastResponse().body.input;
    const match = response.match(/X509Certificate>(.*?)</)
    const result = match ? match[1] : null;

    this.variables.set(certificate, result);
});

Then('det forventede TestCertifikat gemmes som {string}', async function (expectedCertificate) {
    let file = await fs.readFile(`${__dirname}/services/certifikat.txt`);
    this.variables.set(expectedCertificate, file.toString());
});

