const {
    Before,
    BeforeAll,
    After,
    Given,
    Then,
    When,
} = require('@cucumber/cucumber');
const { expect, delay } = require('../util');
const { getPatientDetails } = require('dataaccess/dtg');
const { getPersonWithHealthCareInformation } = require('dataaccess/sces');
const { getNotificationBRS } = require('dataaccess/brs');

const jq = require('node-jq');

Given('registerUpdateFrequency er {int} dage', function (frequency) {
    // Vi kan ikke selv sætte den men konfig er 10 og ændres sjældent så vi tjekker blot at man kalder med 10 eller 62
    expect(frequency).to.be.oneOf([10, 62]);
});

Given(
    'Sundhedsfaglig med ydernr {string} er {string} egen læge for Patient med Cpr {string}',
    //status kan antage værdierne tidligere, nuværende og ikke
    async function (YderNr, Status, PatientCpr) {
        YderNr = this.replaceVariables(YderNr);
        Status = this.replaceVariables(Status);
        PatientCpr = this.replaceVariables(PatientCpr);

        this.SCES = await getPersonWithHealthCareInformation(
            this.attach,
            PatientCpr
        );
        expect(this.SCES.body.code).to.eql(200);

        //DTG historik kaldes så man kan se om hvilke YderNr der tidligere har været registreret som læge for patienten
        this.DTG = await getPatientDetails(this.attach, PatientCpr);

        function DoctorExists(YderNr, DTG) {
            return DTG.body.events.some(function (el) {
                return (
                    el.description ===
                    `Personen fik ændret læge til ydernummer ${YderNr}`
                );
            });
        }

        switch (Status) {
            case 'nuværende':
                expect(
                    this.SCES.body.response.main.Envelope.Body
                        .getPersonWithHealthCareInformationOut
                        .PersonWithHealthCareInformationStructure
                        .PersonHealthCareInformationStructure
                        .CurrentGeneralPractitionerStructure
                        .AssociatedGeneralPractitionerStructure
                        .AssociatedGeneralPractitionerIdentifier._text
                ).to.eql(YderNr);
                break;
            case 'tidligere':
                //Først checkes at læge ikke er nuværende.
                expect(
                    this.SCES.body.response.main.Envelope.Body
                        .getPersonWithHealthCareInformationOut
                        .PersonWithHealthCareInformationStructure
                        .PersonHealthCareInformationStructure
                        .CurrentGeneralPractitionerStructure
                        .AssociatedGeneralPractitionerStructure
                        .AssociatedGeneralPractitionerIdentifier._text
                ).to.not.eql(YderNr);

                //Derefter at den har været tidligere læge jfr DTG historik
                expect(DoctorExists(YderNr, this.DTG)).to.be.true;
                break;
            case 'ikke':
                //Først checkes at læge ikke er nuværende.
                expect(
                    this.SCES.body.response.main.Envelope.Body
                        .getPersonWithHealthCareInformationOut
                        .PersonWithHealthCareInformationStructure
                        .PersonHealthCareInformationStructure
                        .CurrentGeneralPractitionerStructure
                        .AssociatedGeneralPractitionerStructure
                        .AssociatedGeneralPractitionerIdentifier._text
                ).to.not.eql(YderNr);
                //Derefter at den heller ikke har været tidligere læge jfr DTG historik
                expect(DoctorExists(YderNr, this.DTG)).to.be.false;
                break;
            default:
                throw new Error(
                    `Kun værdierne "nuværende", "tidligere" og "ikke" er valide. Modtog: ${Status}`
                );
        }
    }
);

Then('BRS returnerer behandlingsrelation {string}', function (value) {
    // Returnerer BRS service response for aktuel relation
    expect(
        this.getLastResponse().body.response.main.Envelope.Body
            .treatmentRelationResponseBody.ActualRelations.Relation._text
    ).to.eql(value);
});

Then('BRS returnerer feltet {string} lig {string}', function (field, value) {
    // Verifies that content of field is equal to value
    value = this.replaceVariables(value);
    expect(
        this.getLastResponse().body.response.main.Envelope.Body
            .treatmentRelationResponseBody[field]._text
    ).to.eql(value);
});

Given('vi venter {int} millisekunder', { timeout: -1 }, async function (ms) {
    if (ms < 0) {
        throw new Error('Delay must be bigger than 0');
    }
    await delay(ms);
});

Then(
    'der {string} en BRS notification for {string}',
    async function (Status, ExternalReferenceId) {
        ExternalReferenceId = this.replaceVariables(ExternalReferenceId);
        let notifications =
            this.getLastResponse().body.response.main.Envelope.Body
                .NotificationQueryResponseBody.Notifications;
        console.log("this is externalid", ExternalReferenceId);

        switch (Status) {
            case 'findes':
                let notification = await jq.run(
                    `if(.[]|select(.ExternalReferenceId._text=="${ExternalReferenceId}")) then (true) else (false) end`,
                    notifications,
                    { input: 'json', output: 'json' }
                );
                expect(notification).to.be.true;
                break;
            default:
                break;
        }
    }
);

Then(
    'BRS notification indeholder behandlingsrelation {string}',
    async function (TreatmentRelation) {
        let notifications =
            this.getLastResponse().body.response.main.Envelope.Body
                .NotificationQueryResponseBody.Notifications;
        let treatmentRelation = await jq.run(
            `.[]|select(.TreatmentRelationAlarmType.TreatmentRelationFollowup.AcceptableRelations.Relation._text=="${TreatmentRelation}")`,
            notifications,
            { input: 'json', output: 'json' }
        );
        expect(
            treatmentRelation.TreatmentRelationAlarmType
                .TreatmentRelationFollowup.AcceptableRelations.Relation._text
        ).to.eql(TreatmentRelation);
    }
);

Then('største serienummer udskrives', function () {
    this.attach('serienummer');
});


Given('{string} er den sidste', async function (serienummer) {
    var serialnumber = 2420125
    let response = await getNotificationBRS(this.attach, String(serialnumber));

    var last = serialnumber + response.body.response.main.Envelope.Body.NotificationQueryResponseBody.Notifications.length;
    console.log("sidste serienummer er: " + last) //Kan max se 100 frem
    this.variables.set(serienummer, String(last));
});

Then(
    'der findes en BRS notification for {string}',
    async function (ExternalReferenceId) {
        ExternalReferenceId = this.replaceVariables(ExternalReferenceId);
        let notifications =
            this.getLastResponse().body.response.main.Envelope.Body
                .NotificationQueryResponseBody.Notifications;
        console.log(notifications)
        console.log("this is externalid", ExternalReferenceId);

        console.log(notifications.ExternalReferenceId._text)
        expect(notifications.ExternalReferenceId._text).to.eql(ExternalReferenceId);
        //expect(notifications.ExternalReferenceId).to.eql(ExternalReferenceId);
    }
);

Then('BRS returnerer RelationBySource LPR {string}, LPR3 {string}, Henvisninger {string}, SSR {string} og Sikrede {string}',
    function (LPR, LPR3, Henvisninger, SSR, Sikrede) {
        console.log(JSON.stringify(expect(this.getLastResponse().body.response.main.Envelope.Body.treatmentRelationResponseBody.RelationsBySources.RelationBySource[0].Relation._text).to.eql(LPR),null,2));
        console.log(JSON.stringify(expect(this.getLastResponse().body.response.main.Envelope.Body.treatmentRelationResponseBody.RelationsBySources.RelationBySource[1].Relation._text).to.eql(LPR3),null,2));
        console.log(JSON.stringify(expect(this.getLastResponse().body.response.main.Envelope.Body.treatmentRelationResponseBody.RelationsBySources.RelationBySource[2].Relation._text).to.eql(Henvisninger),null,2));
        console.log(JSON.stringify(expect(this.getLastResponse().body.response.main.Envelope.Body.treatmentRelationResponseBody.RelationsBySources.RelationBySource[3].Relation._text).to.eql(SSR),null,2));
        console.log(JSON.stringify(expect(this.getLastResponse().body.response.main.Envelope.Body.treatmentRelationResponseBody.RelationsBySources.RelationBySource[4].Relation._text).to.eql(Sikrede),null,2));

    }
);
