const {
    Before,
    BeforeAll,
    After,
    Given,
    Then,
    When,
} = require('@cucumber/cucumber');
const jq = require('node-jq');
const { v4 } = require('uuid');
const { expect } = require('../util');
const {
    ddsregistry: { getPDCEntries },
    skr: {
        GetPersonalDataCard,
        DeleteRelatives,
        DeleteRelativesHP,
        CreateRelative,
        CreateTemporaryAddress,
        DeleteTemporaryAddress,
        CreateHealthProvider,
        DeleteHealthProvider,
        CreateLanguage,
        DeleteLanguage,
        UpdateContactInformationInvalidCpr,
        GetPersonalDataCardInvalidCpr,
    },
} = require('dataaccess');
const { getPatientDetails } = require('dataaccess/dtg');

//#region FSK Datacard
// This fixture expects personal data card to contain pdc component id and pushing the component
//Added timeout, because NRT-2501 kept timing out in creating relative
Then(
    '{string} findes i FSK stamkortet for {string}',
    { timeout: -1 },
    async function (PDCComponentId, PatientCPR) {
        // Replace CPR
        PatientCPR = this.replaceVariables(PatientCPR);

        // Replace PDC component id
        PDCComponentId = this.replaceVariables(PDCComponentId);

        // Getting the PDC entries
        let entrysfskRepositry = await getPDCEntries(this.attach, PatientCPR, 3);

        // Finding component with pdc component id
        const pdcComponent = await jq.run(
            `.[]|select(.observation.id.__extension=="${PDCComponentId}")`,
            entrysfskRepositry,
            { input: 'json', output: 'json' }
        );

        // expecting the pdc component id not to be null
        expect(pdcComponent?.observation?.id?.__extension).to.not.be.null;

        // Pushing the pdc component
        this.documents.push(pdcComponent);
        console.log(pdcComponent)
    }
);

//udkommenteret grundet at det ikke er muligt at hente gennem DDS længere
/*Then(
    '{string} findes i FSK stamkortet version 3 for {string}',
    { timeout: -1 },
    async function (PDCComponentId, PatientCPR) {
        PatientCPR = this.replaceVariables(PatientCPR);
        PDCComponentId = this.replaceVariables(PDCComponentId);

        let entryDdsRepositry = await getPDCEntries(this.attach, PatientCPR, 3);
        const pdcComponent = await jq.run(
            `.[]|select(.observation.id.__extension=="${PDCComponentId}")`,
            entryDdsRepositry,
            { input: 'json', output: 'json' }
        );

        expect(pdcComponent?.observation?.id?.__extension).to.not.be.null;
        this.documents.push(pdcComponent);
    }
);*/


// This fixture expects PDC component to not exist in DDS repository entries
Then(
    '{string} findes ikke i FSK stamkortet for {string}',
    { timeout: -1 },
    async function (PDCComponentId, PatientCPR) {
        // Replace CPR
        PatientCPR = this.replaceVariables(PatientCPR);

        // Getting the PDC entries
        let entry_sfskRepositry = await getPDCEntries(this.attach, PatientCPR, 3);

        // Expecting the DDS repository entries not to contain PDC component id
        expect(entry_sfskRepositry).not.to.contain(PDCComponentId);
    }
);
//#endregion

//#region SKR Datacard
// This fixture expects pdc component with pdc id to exist in SKR data card
Then(
    '{string} med {string} findes i SKR stamkortet for {string}',
    { timeout: -1 },
    async function (PDCComponent, PDCComponentId, PatientCPR) {
        // Replace CPR
        PatientCPR = this.replaceVariables(PatientCPR);

        // Replace pdc component id
        PDCComponentId = this.replaceVariables(PDCComponentId);

        // Getting personal data card
        const personalDataCardEntry = await GetPersonalDataCard(
            this.attach,
            PatientCPR
        );

        if (Array.isArray(personalDataCardEntry)) {
            // Switch case: Expects component to either be contain language, temporary address, related person or health provider with given id
            switch (PDCComponent) {
                case 'sprog':
                    // Finding pdc component with language
                    const pdcComponentLanguage = await jq.run(
                        `.[]|select(.language)`,
                        personalDataCardEntry,
                        { input: 'json', output: 'json' }
                    );

                    // Expecting the language id to equal pdc compoent id
                    expect(pdcComponentLanguage.language.id.__extension).to.eql(
                        PDCComponentId
                    );

                    // Pushing language pdc component
                    this.documents.push(pdcComponentLanguage);
                    break;
                case 'pårørende':
                    // Finding pdc component with relative
                    const pdcComponentRelative = await jq.run(
                        `.[]|select(.relatedPerson.id.__extension=="${PDCComponentId}")`,
                        personalDataCardEntry,
                        { input: 'json', output: 'json' }
                    );

                    // Expecting the relative id to equal pdc component id
                    expect(
                        pdcComponentRelative.relatedPerson.id.__extension
                    ).to.eql(PDCComponentId);

                    // Pushing relative pdc component
                    this.documents.push(pdcComponentRelative);
                    break;
                case 'midlertidig adresse':
                    // Finding temporary address pdc component
                    let pdcComponentTempAddress = await jq.run(
                        `.[]|select(.temporaryAddress)`,
                        personalDataCardEntry,
                        { input: 'json', output: 'json' }
                    );

                    // Expecting the temporary address id to equal pdc component id
                    expect(
                        pdcComponentTempAddress.temporaryAddress.id.__extension
                    ).to.eql(PDCComponentId);

                    // Pushing temporary address pdc component
                    this.documents.push(pdcComponentTempAddress);
                    break;
                case 'tandlæge':
                    // Finding health provider pdc component
                    let pdcComponentHealthProvider = await jq.run(
                        `.[]|select(.healthProvider)`,
                        personalDataCardEntry,
                        { input: 'json', output: 'json' }
                    );

                    // Expecting health provider id to equal pdc component id
                    expect(
                        pdcComponentHealthProvider.healthProvider.id.__extension
                    ).to.eql(PDCComponentId);

                    // Pushing health provider pdc component
                    this.documents.push(pdcComponentHealthProvider);
                    break;
                default:
                    break;
            }
        } else {
            switch (PDCComponent) {
                case 'sprog':
                    // Expecting the language id to equal pdc compoent id
                    expect(personalDataCardEntry.language.id.__extension).to.eql(
                        PDCComponentId
                    );
                    break;
                case 'pårørende':
                    // Expecting the relative id to equal pdc component id
                    expect(
                        personalDataCardEntry.relatedPerson.id.__extension
                    ).to.eql(PDCComponentId);
                    break;
                case 'midlertidig adresse':
                    // Expecting the temporary address id to equal pdc component id
                    expect(
                        personalDataCardEntry.temporaryAddress.id.__extension
                    ).to.eql(PDCComponentId);
                    break;
                case 'tandlæge':
                    // Expecting health provider id to equal pdc component id
                    expect(
                        personalDataCardEntry.healthProvider.id.__extension
                    ).to.eql(PDCComponentId);
                    break;
                default:
                    break;
            }
            this.documents.push(personalDataCardEntry);
        }
    }
);
//#endregion

//#region General - Language, Relative, Temporary Address and Health Provider
// This fixture makes sure that the pdc component (language, temporary address, related person and health provider) exist in the personal data card and saves the id of the pdc componet
Given(
    '{string} findes på {string} stamkort og id gemmes som {string}',
    { timeout: -1 },
    async function (PDCComponent, PatientCPR, PDCComponentId) {
        // Replace CPR
        PatientCPR = this.replaceVariables(PatientCPR);

        // Getting PDC entries
        let entry_sfskRepositry = await getPDCEntries(this.attach, PatientCPR, 3);
        console.log('Entry data:', JSON.stringify(entry_sfskRepositry, null, 2));
        /* // Finding relative pdc component
         const pdcComponent = await jq.run(
             `.[]|select(.observation.code.__code=="RelativeTypedIn")`,
             entry_sfskRepositry,
             { input: 'json', output: 'json' }
         );*/

        // Switch case: Expects component to either be contain language, temporary address, related person or health provider and pushes the pdc component id
        switch (PDCComponent) {
            case 'sprog':
                // Finding language pdc component
                const pdcComponentLanguage = await jq.run(
                    `.[]|select(.observation.value.__codeSystem? =="1.0.639.1")`,
                    entry_sfskRepositry,
                    { input: 'json', output: 'json' }
                );
                console.log(JSON.stringify(pdcComponentLanguage))
                // Setting pdc component id with found language pdc component id
                this.variables.set(
                    PDCComponentId,
                    pdcComponentLanguage.observation.id.__extension
                );

                // Pushing language pdc component
                this.documents.push(pdcComponentLanguage);
                break;
            case 'pårørende':
                // Finding relative pdc component
                const pdcComponentRelative = await jq.run(
                    `[.[]|select(.observation.code|select(."__code"? =="RelativeTypedIn")).observation.id."__extension"]`,
                    entry_sfskRepositry,
                    { input: 'json', output: 'json' }
                );
                console.log(JSON.stringify(pdcComponentRelative, null, 2));

                // If zero relatives
                if (pdcComponentRelative.length == 0) {
                    // Making unique relative id
                    let RelativeId = v4();

                    // Creating relative
                    await CreateRelative(this.attach, PatientCPR, RelativeId);

                    // Setting pdc component id variable with the new relative id
                    this.variables.set(PDCComponentId, RelativeId);
                } else {
                    console.log("flere relatives");
                    // Else - one or more relatives
                    // Setting pdc component id variable with the first relative id
                    // Ændret denne linje:
                    this.variables.set(PDCComponentId, pdcComponentRelative[0]);
                }

                // Pushing relative pdc component
                this.documents.push(pdcComponentRelative);
                break;
            case 'tandlæge':
                // Finding health provider pdc component
                const pdcComponentHealthProvider = await jq.run(
                    `.[]|select(.observation.code."__code"? =="DentistTypedIn")`,
                    entry_sfskRepositry,
                    { input: 'json', output: 'json' }
                );
                // If health provider doesn't exist
                if (pdcComponentHealthProvider == '') {
                    // Generating new guid id
                    let HealthProviderId = v4();

                    // Creating health provider
                    await CreateHealthProvider(
                        this.attach,
                        HealthProviderId,
                        PatientCPR
                    );
                    // Setting pdc component id variable with the new health provider id
                    this.variables.set(PDCComponentId, HealthProviderId);
                } else {
                    // Health provider exists
                    // Setting pdc component id variable to health provider id
                    this.variables.set(
                        PDCComponentId,
                        pdcComponentHealthProvider.observation.id.__extension
                    );
                }
                // Pushing health provider pdc component
                this.documents.push(pdcComponentHealthProvider);
                break;
            case 'midlertidig adresse':
                // Finding temporary address pdc component
                const pdcComponentTempAddress = await jq.run(
                    `.[]|select(.observation.code."__code"?=="TempAddrTypedIn")`,
                    entry_sfskRepositry,
                    { input: 'json', output: 'json' }
                );

                // If temporary address doesn't exist
                if (pdcComponentTempAddress == '') {
                    // Generating new guid id
                    let TempAdressId = v4();

                    // Creating new temporary address
                    await CreateTemporaryAddress(
                        this.attach,
                        TempAdressId,
                        PatientCPR
                    );

                    // Setting pdc component id variable to the new temporary id
                    this.variables.set(PDCComponentId, TempAdressId);
                } else {
                    // Temporary address exists
                    // Setting pdc component id variable to temporary id
                    this.variables.set(
                        PDCComponentId,
                        pdcComponentTempAddress.observation.id.__extension
                    );
                }
                // Pushing temporary address pdc component
                this.documents.push(pdcComponentTempAddress);
                break;
            case 'kontaktoplysninger':
                const pdcComponentPatientContact = await jq.run(
                    `.[]|select(.observation.code."__code"?=="PatientContactTypedIn")`,
                    entry_sfskRepositry,
                    { input: 'json', output: 'json' }
                );
                expect(pdcComponentPatientContact).to.not.eql(null);

                this.variables.set(
                    PDCComponentId,
                    pdcComponentPatientContact.observation.id.__extension
                );

                this.documents.push(pdcComponentPatientContact);
                break;
            default:
                // Throwing error if unknown pdc component is given in the fixture
                throw new Error(`Unknown pdc component "${PDCComponent}"`);
        }
    }
);
//#endregion

//#region Relative Fixtures
// This fixture deletes relatives if a relative exists
Given(
    'pårørende {string} findes ikke på {string} stamkort i forvejen',
    { timeout: -1 },
    async function (PaaroerendeId, PatientCPR) {
        // Replacing CPR
        PatientCPR = this.replaceVariables(PatientCPR);

        // Replacing relative id
        PaaroerendeId = this.replaceVariables(PaaroerendeId);

        // Getting PDC
        let datacard = await GetPersonalDataCard(this.attach, PatientCPR);

        // If PDC exists
        if (datacard != undefined) {
            // Finding related person pdc component with relative id
            const res = await jq.run(
                `.. | select(.relatedPerson?.id?.__extension == "${PaaroerendeId}")`,
                datacard,
                { input: 'json', output: 'json' }
            );

            // If related person exists
            if (res.relatedPerson?.id.__extension == PaaroerendeId) {
                // Deleting relative
                await DeleteRelatives(this.attach, PatientCPR, PaaroerendeId);
            }
        }
    }
);

// This fixtures deletes an amount of relatives on a citizen
Given(
    'slet {string} pårørende på {string}',
    { timeout: -1 },
    async function (Amount, PatientCPR) {
        // Replacing CPR
        PatientCPR = this.replaceVariables(PatientCPR);

        // Getting PDC
        let datacard = await GetPersonalDataCard(this.attach, PatientCPR);

        // If PDC exists
        if (datacard != undefined) {
            // Finding related person pdc component with relative id
            const res = await jq.run(
                `[.[] | select(.relatedPerson)]`,
                datacard,
                { input: 'json', output: 'json' }
            );
            if (res.length != 0) {
                // All relatives must be deleted
                if (Amount == 'alle') {
                    for (let i = 0; i < res.length; i++) {
                        const id = res[i].relatedPerson.id.__extension;
                        await DeleteRelatives(this.attach, PatientCPR, id);
                    }
                } else {
                    // A number of relatives must be deleted
                    Amount = parseInt(Amount);
                    for (let i = 0; i < Amount; i++) {
                        const id = res[i].relatedPerson.id.__extension;
                        await DeleteRelatives(this.attach, PatientCPR, id);
                    }
                }
            } else {
                this.attach('Ingen pårørende');
            }
        }
    }
);

// This fixture expects related person data in data table to equal data in PDC (FSK)
Then(
    'pårørende med {string} har datasæt {string} i FSK stamkortet',
    async function (RelatedPersonId, DataSaet) {
        // Getting data table
        const dataset = this.datasets.get(DataSaet);

        // Getting pdc component from last document
        const pdcComponent = this.getLastDocument();

        // Related Person Id
        RelatedPersonId = this.replaceVariables(RelatedPersonId);

        // Finding related person id
        let ResponseRelatedPersonId = pdcComponent.observation.id.__extension;

        expect(ResponseRelatedPersonId).to.eql(RelatedPersonId);

        // Expecting data from data table to equal data from PDC
        // Person Name
        let PN = await jq.run(
            `.observation.value[]|select(."__{http://www.w3.org/2001/XMLSchema-instance}type"=="PN")`,
            pdcComponent,
            { input: 'json', output: 'json' }
        );

        if (dataset.relatedPersonGivenName.includes('&')) {
            expect(PN.given._text).to.eql(' ');
        } else {
            expect(PN.given._text).to.eql(dataset.relatedPersonGivenName);
        }

        // Related Person Name
        if (dataset.relatedPersonFamilyName == undefined) {
            expect(PN.family._text).to.eql(dataset.relatedPersonFamilyName);
        } else if (dataset.relatedPersonFamilyName.includes('&')) {
            expect(PN.family._text).to.eql(' ');
        }

        // Realtionship Type
        let relType = await jq.run(
            `.observation.value[]|select(.__codeSystem=="1.2.208.184.100.2")`,
            pdcComponent,
            { input: 'json', output: 'json' }
        );

        expect(relType.__code).to.eql(dataset.relationshipType);

        // Phone Home
        let phoneHome = await jq.run(
            `.observation.value[]|select(."__{http://www.w3.org/2001/XMLSchema-instance}type"=="TEL" and .__use=="H")`,
            pdcComponent,
            { input: 'json', output: 'json' }
        );

        if (dataset.phoneH == '') {
            expect(phoneHome.__value).to.eql(undefined);
        } else {
            expect(phoneHome.__value).to.eql('tel:' + dataset.phoneH);
        }

        // Phone Work
        let phoneWork = await jq.run(
            `.observation.value[]|select(."__{http://www.w3.org/2001/XMLSchema-instance}type"=="TEL" and .__use=="WP")`,
            pdcComponent,
            { input: 'json', output: 'json' }
        );

        if (dataset.phoneWP == '') {
            expect(phoneWork.__value).to.eql(undefined);
        } else {
            expect(phoneWork.__value).to.eql('tel:' + dataset.phoneWP);
        }

        // Phone Mobile
        let phoneMobile = await jq.run(
            `.observation.value[]|select(."__{http://www.w3.org/2001/XMLSchema-instance}type"=="TEL" and .__use=="MC")`,
            pdcComponent,
            { input: 'json', output: 'json' }
        );

        if (dataset.phoneMC == '') {
            expect(phoneMobile.__value).to.eql(undefined);
        } else {
            expect(phoneMobile.__value).to.eql('tel:' + dataset.phoneMC);
        }
    }
);

Then(
    'pårørende med {string} er slettet i SKR for {string} af medarbejder',
    async function (RelatedPersonId, PatientCPR) {
        await DeleteRelativesHP(this.attach, RelatedPersonId, PatientCPR)
    }
);

// This fixture expects related person data in data table to equal data in PDC (SKR)
Then(
    'pårørende med {string} har datasæt {string} i SKR stamkortet',
    async function (RelativeId, Dataset) {
        // Getting data table
        const dataset = this.datasets.get(Dataset);

        // Getting pdc component from last document
        const pdcComponent = this.getLastDocument();

        // Relative Id
        RelativeId = this.replaceVariables(RelativeId);
        let ResponseRelativeId = pdcComponent.relatedPerson.id.__extension;

        expect(ResponseRelativeId).to.eql(RelativeId);

        // Expecting data from data table to equal data from PDC
        // Person Given Name
        if (dataset.relatedPersonGivenName != undefined) {
            let relatedPersonGivenName =
                pdcComponent.relatedPerson.associatedEntity.associatedPerson
                    .name.given._text;

            expect(relatedPersonGivenName).to.eql(
                dataset.relatedPersonGivenName
            );
        }

        // Person Family Name
        if (dataset.relatedPersonFamilyName != undefined) {
            let relatedPersonFamilyName =
                pdcComponent.relatedPerson.associatedEntity.associatedPerson
                    .name.family._text;

            expect(relatedPersonFamilyName).to.eql(
                dataset.relatedPersonFamilyName
            );
        }

        // Relationship Type
        if (dataset.relationshipType != undefined) {
            let relationshipType =
                pdcComponent.relatedPerson.relationshipType.__code;

            expect(relationshipType).to.eql(dataset.relationshipType);
        }

        // Phone Home
        if (dataset.phoneH != undefined) {
            let phoneHome = await jq.run(
                `.relatedPerson.associatedEntity.telecom[]|select(.__use=="H")`,
                pdcComponent,
                { input: 'json', output: 'json' }
            );

            if (dataset.phoneH == '') {
                expect(phoneHome.__value).to.eql(undefined);
            } else {
                expect(phoneHome.__value).to.eql('tel:' + dataset.phoneH);
            }
        }

        // Phone Work
        if (dataset.phoneWP != undefined) {
            let phoneWork = await jq.run(
                `.relatedPerson.associatedEntity.telecom[]|select(.__use=="WP")`,
                pdcComponent,
                { input: 'json', output: 'json' }
            );

            if (dataset.phoneWP == '') {
                expect(phoneWork.__value).to.eql(undefined);
            } else {
                expect(phoneWork.__value).to.eql('tel:' + dataset.phoneWP);
            }
        }
        // Phone Mobile
        if (dataset.phoneMC != undefined) {
            let phoneMobile = await jq.run(
                `.relatedPerson.associatedEntity.telecom[]|select(.__use=="MC")`,
                pdcComponent,
                { input: 'json', output: 'json' }
            );

            if (dataset.phoneMC == '') {
                expect(phoneMobile.__value).to.eql(undefined);
            } else {
                expect(phoneMobile.__value).to.eql('tel:' + dataset.phoneMC);
            }
        }

        // Related Person Address
        if (dataset.customAddress == 'Ja') {
            let streetAddressLine =
                pdcComponent.relatedPerson.associatedEntity.addr
                    .streetAddressLine._text;
            let postalCode =
                pdcComponent.relatedPerson.associatedEntity.addr.postalCode
                    ._text;
            let city =
                pdcComponent.relatedPerson.associatedEntity.addr.city._text;
            let country =
                pdcComponent.relatedPerson.associatedEntity.addr.country._text;

            expect(streetAddressLine).to.eql(dataset.streetAddressLine);
            expect(postalCode).to.eql(dataset.postalCode);
            expect(city).to.eql(dataset.city);
            expect(country).to.eql(dataset.country);
        }
    }
);

// This fixture makes sure that relative exists or not exists based on the given status
Given(
    'pårørende {string} på {string} stamkort',
    async function (Status, PatientCPR) {
        // Replacing CPR
        PatientCPR = this.replaceVariables(PatientCPR);

        // Getting PDC entries
        let entry_sfskRepositry = await getPDCEntries(this.attach, PatientCPR, 3);

        // Finding relative pdc component
        const pdcComponent = await jq.run(
            `.[]|select(.observation.code.__code=="RelativeTypedIn")`,
            entry_sfskRepositry,
            { input: 'json', output: 'json' }
        );

        // Switch case based on status
        switch (Status) {
            // Status is existing, therefore related person must exist
            case 'findes':
                // If related person does not exist
                if (pdcComponent == '') {
                    // Generating new guid id
                    let RelatedPersonId = v4();

                    // Creating related person
                    await CreateRelative(
                        this.attach,
                        PatientCPR,
                        RelatedPersonId
                    );
                }
                break;
            // Status is not existing, therefore related person must NOT exist
            case 'findes ikke':
                // If related person exists
                if (pdcComponent != '') {
                    // Finding related person id
                    RelatedPersonId = pdcComponent.observation.id.__extension;

                    // Deleting relative
                    await DeleteRelatives(
                        this.attach,
                        PatientCPR,
                        RelatedPersonId
                    );
                }
                break;
            default:
                // Throwing error if status not exists
                throw new Error(`Unknown status "${Status}"`);
        }
        // Pushing relative pdc component
        this.documents.push(pdcComponent);
    }
);
//#endregion

//#region Language Fixtures
//#region FSK Datacard
// This fixture expects pdc component to equal values in data table
Then(
    'sprog med {string} har datasæt {string} i FSK stamkortet',
    async function (LanguageId, Datasaet) {
        // Getting data table
        const dataset = this.datasets.get(Datasaet);

        // Getting pdc component from last document
        const pdcComponent = this.getLastDocument();

        // Expecting values from data table to equal pdc component values
        // Landuage Id
        LanguageId = this.replaceVariables(LanguageId);
        let ResponseLanguageId = pdcComponent.observation.id.__extension;

        expect(ResponseLanguageId).to.eql(LanguageId);

        // Language Code
        let language = await jq.run(
            `.observation.value|select(.__codeSystem=="1.0.639.1")`,
            pdcComponent,
            { input: 'json', output: 'json' }
        );

        let data = {
            'languageCode': language.__code,
            'codeSystemName': language.__codeSystemName,
            'displayName': language.__displayName
        }

        for (let [key, value] of Object.entries(dataset)) {
            if (key in data) {
                try {
                    expect(value).to.equal(data[key]);
                } catch (error) {
                    console.log(value + " var i stedet lig med " + data[key]);
                    throw error;
                }
            }
        }

    }
);
//#endregion

//#region SKR Datacard
// This fixture expects data table to equal values in language pdc component
Then(
    'sprog med {string} har datasæt {string} i SKR stamkortet',
    async function (LanguageId, Datasaet) {
        // Replacing language id
        LanguageId = this.replaceVariables(LanguageId);

        // Getting data table
        const dataset = this.datasets.get(Datasaet);

        // Getting pdc component
        const pdcComponent = this.getLastDocument();

        // Expecting data table to equal values in language pdc component
        //Language Id
        LanguageId = this.replaceVariables(LanguageId);
        let ResponseLanguageId = pdcComponent.language.id.__extension;

        expect(ResponseLanguageId).to.eql(LanguageId);

        //Language Code
        expect(pdcComponent.language.languageCode._text).to.eql(
            dataset.languageCode
        );
    }
);
//#endregion

//#region FSK and SKR
// This fixture makes sure that language exists or not exists based on the given status
Given(
    'sprog {string} på {string} stamkort',
    { timeout: -1 },
    async function (Status, PatientCPR) {
        // Replacing CPR
        PatientCPR = this.replaceVariables(PatientCPR);

        // Getting pdc documents/entries
        let entry_sfskRepositry = await getPDCEntries(this.attach, PatientCPR, 3);

        // Finding language pdc component
        const pdcComponent = await jq.run(
            `.[]|select(.observation.code.__code=="LanguageTypedIn")`,
            entry_sfskRepositry,
            { input: 'json', output: 'json' }
        );

        // Switch case based on status
        switch (Status) {
            // Case language exists
            case 'findes':
                // If language pdc compont doesn't exist
                if (pdcComponent == '') {
                    // Generating language id
                    let LanguageId = v4();

                    // Creating language
                    await CreateLanguage(this.attach, LanguageId, PatientCPR);
                }
                break;
            // Case language does not exist
            case 'findes ikke':
                // If language pdc component exist
                if (pdcComponent != '') {
                    // Finding language id in pdc component
                    LanguageId = pdcComponent.observation.id.__extension;

                    // Deleting language
                    await DeleteLanguage(this.attach, LanguageId, PatientCPR);
                }
                break;
            default:
                // Throwing error if status is unknown
                throw new Error(`Unknown status "${Status}"`);
        }

        // Pushing pdc component
        this.documents.push(pdcComponent);
    }
);
//#endregion
//#endregion

//#region Health Provider Fixtures
//#region FSK and SKR Datacard
// This fixture makes sure that health provider exists or not exists based on the given status
Given(
    'tandlæge {string} på {string} stamkort',
    { timeout: -1 },
    async function (Status, PatientCPR) {
        // Replacing CPR
        PatientCPR = this.replaceVariables(PatientCPR);

        // Getting pdc documents/entries
        let entry_sfskRepositry = await getPDCEntries(this.attach, PatientCPR, 3);

        // Finding health provider pdc component
        let pdcComponent = await jq.run(
            `.[]|select(.observation.code.__code=="DentistTypedIn")`,
            entry_sfskRepositry,
            { input: 'json', output: 'json' }
        );

        // Switch case based on status
        switch (Status) {
            // Case health provider exists
            case 'findes':
                // If health provider pdc compont doesn't exist
                if (pdcComponent == '') {
                    // Generating health provider id
                    let HealthProviderId = v4();

                    // Creating health provider
                    await CreateHealthProvider(
                        this.attach,
                        HealthProviderId,
                        PatientCPR
                    );
                }
                break;
            // Case health provider does not exist
            case 'findes ikke':
                console.log("1");
                // If health provider pdc component exist
                if (pdcComponent != '') {
                    // Finding health provider id
                    HealtProviderId = pdcComponent.observation.id.__extension;
                    console.log("2");
                    // Deleting health provider
                    await DeleteHealthProvider(
                        this.attach,
                        HealtProviderId,
                        PatientCPR
                    );
                }
                break;
            default:
                // Throwing error if status is unknown
                throw new Error(`Unknown status "${Status}"`);
        }
        // Pushing pdc component
        this.documents.push(pdcComponent);
    }
);
//#endregion

//#region FSK Datacard
// This fixture expects data table to equal values in health provider pdc component
Then(
    'tandlæge med {string} har datasæt {string} i FSK stamkortet',
    { timeout: -1 },
    async function (HealthProviderId, Datasaet) {
        // Getting data table
        const dataset = this.datasets.get(Datasaet);

        // Getting pdc component from last document
        const pdcComponent = this.getLastDocument();
        console.log(pdcComponent);
        console.log(pdcComponent.observation.value[1])
        // Expecting values in data table to equal data in pdc component
        // Health Provider Id
        HealthProviderId = this.replaceVariables(HealthProviderId);
        ResponseHealthProviderId = pdcComponent.observation.id.__extension;

        expect(ResponseHealthProviderId).to.eql(HealthProviderId);

        // YderNumber
        let yderNumber = await jq.run(
            `.observation.value[]|select(."__{http://www.w3.org/2001/XMLSchema-instance}type"=="II")`,
            pdcComponent,
            { input: 'json', output: 'json' }
        );

        expect(yderNumber.__extension).to.eql(dataset.yderNumber);

        // YderName
        let yderName = await jq.run(
            `.observation.value[]|select(."__{http://www.w3.org/2001/XMLSchema-instance}type"=="EN")`,
            pdcComponent,
            { input: 'json', output: 'json' }
        );
        expect(yderName.given._text).to.eql(dataset.yderName);

        console.log(pdcComponent)
        if (dataset.customAddress == 'Ja') {
            // Street Address
            let address = await jq.run(
                `.observation.value[]|select(."__{http://www.w3.org/2001/XMLSchema-instance}type"=="AD")`,
                pdcComponent,
                { input: 'json', output: 'json' }
            );
            expect(address.streetAddressLine._text).to.eql(
                dataset.streetAddressLine
            );
            expect(address.postalCode._text).to.eql(dataset.postalCode);
            expect(address.city._text).to.eql(dataset.city);
            expect(address.country._text).to.eql(dataset.country);
        }

        // Work Phone
        let workPhone = await jq.run(
            `.observation.value[]|select(."__{http://www.w3.org/2001/XMLSchema-instance}type"=="TEL" and .__use=="WP")`,
            pdcComponent,
            { input: 'json', output: 'json' }
        );

        if (dataset.phoneWP == '' || dataset.phoneWP == null) {
            expect(workPhone.__value).to.eql('tel:null');
        } else {
            expect(workPhone.__value).to.eql('tel:' + dataset.phoneWP);
        }
    }
);

Then(
    'tandlæge med {string} har datasæt {string} i FSK stamkortet version 3',
    { timeout: -1 },
    async function (HealthProviderId, Datasaet) {
        const dataset = this.datasets.get(Datasaet);
        const pdcComponent = this.getLastDocument();
        console.log(pdcComponent);

        HealthProviderId = this.replaceVariables(HealthProviderId);
        ResponseHealthProviderId = pdcComponent.observation.id.__extension;

        expect(ResponseHealthProviderId).to.eql(HealthProviderId);
        // YderNumber
        let yderNumber = await jq.run(
            `.observation.value[]|select(."__{http://www.w3.org/2001/XMLSchema-instance}type"=="II")`,
            pdcComponent,
            { input: 'json', output: 'json' }
        );
        expect(yderNumber.__extension).to.eql(dataset.yderNumber);

        // YderName
        let yderName = await jq.run(
            `.observation.value[] | select(."__{http://www.w3.org/2001/XMLSchema-instance}type" == "TN")`,
            pdcComponent,
            { input: 'json', output: 'json' }
        );
        expect(yderName._text).to.eql(dataset.yderName);

        let address = await jq.run(
            `.observation.value[]|select(."__{http://www.w3.org/2001/XMLSchema-instance}type"=="AD")`,
            pdcComponent,
            { input: 'json', output: 'json' }
        );
        expect(address.streetAddressLine._text).to.eql(
            dataset.streetAddressLine
        );
        expect(address.postalCode._text).to.eql(dataset.postalCode);
        expect(address.city._text).to.eql(dataset.city);
        expect(address.country._text).to.eql(dataset.country);

        // Work Phone
        let workPhone = await jq.run(
            `.observation.value[]|select(."__{http://www.w3.org/2001/XMLSchema-instance}type"=="TEL" and .__use=="WP")`,
            pdcComponent,
            { input: 'json', output: 'json' }
        );
        expect(workPhone.__value).to.eql('tel:' + dataset.phoneWP);
    }
);

Then('{string} indeholder tandlaege med datasæt {string}', function (responseName, dataset) {
    const expectedDataset = this.datasets.get(dataset);
    let dentist = JSON.parse(this.variables.get(responseName).text).ClinicalDocument.dentist;
    let data = {
        'codeSystem': dentist.code.codeSystem,
        'codeSystemName': dentist.code.codeSystemName,
        'displayName': dentist.code.displayName,
        'root': dentist.id.root,
        'assigningAuthorityName': dentist.identification.authorityName,
        'extension': dentist.identification.extension,
        'identificationRoot': dentist.identification.root
    }

    for (k of Object.entries(expectedDataset)) {
        try {
            expect(k[1]).to.equal(data[k[0]]);
        } catch (error) {
            console.log(k[0] + " var i stedet lig med " + data[k[0]]);
            throw error;
        }
    }

});

Then('{string} indeholder paaroerende {string} med datasæt {string}', function (response, PaaroerendeId, dataset) {
    const expectedDataset = this.datasets.get(dataset);
    PaaroerendeId = this.replaceVariables(PaaroerendeId);
    let relatives = JSON.parse(this.variables.get(response).text).ClinicalDocument.relatives;
    let count = 0;
    let relative;
    for (r of relatives) {
        if (r.id.extension = PaaroerendeId) {
            relative = relatives[count]
        }
        count++
    }

    let data = {
        'codeSystem': relative.code.codeSystem,
        'codeSystemName': relative.code.codeSystemName,
        'displayName': relative.code.displayName,
        'root': relative.id.root,
        'code': relative.code.code,
    }

    for (k of Object.entries(expectedDataset)) {
        try {
            expect(k[1]).to.equal(data[k[0]]);
        } catch (error) {
            console.log(k[1] + " var i stedet lig med " + data[k[0]]);
            throw error;
        }
    }
});

Then('registersvar {string} indeholder {string} med stamkortid {string}', function (response, documentName, pdcID) {
    let savedResponse = this.variables.get(response).body.response.main.Envelope.Body
        .AdhocQueryResponse.RegistryObjectList;
    expect(savedResponse.ExtrinsicObject.Name.LocalizedString.__value).to.equal(documentName)
    expect(savedResponse.ExtrinsicObject.__id).to.equal(pdcID)
});
//#endregion

//#region SKR Datacard
// This fixture expects data table to equal values in health provider pdc component
Then(
    'tandlæge med {string} har datasæt {string} i SKR stamkortet',
    async function (HealthProviderId, Datasaet) {
        //Getting data table
        const dataset = this.datasets.get(Datasaet);

        // Getting pdc component from last document
        const pdcComponent = this.getLastDocument();

        // Expecting values in data table to equal data in pdc component
        // Health Provider Id
        HealthProviderId = this.replaceVariables(HealthProviderId);
        let ResponseHealthProviderId =
            pdcComponent.healthProvider.id.__extension;

        expect(ResponseHealthProviderId).to.eql(HealthProviderId);

        // YderNumber
        if (dataset.yderNumber != null) {
            let yderNumber =
                pdcComponent.healthProvider.organization.id.__extension;
            expect(yderNumber).to.eql(dataset.yderNumber);
        }

        // YderName
        if (dataset.yderName != null) {
            let yderName = pdcComponent.healthProvider.organization.name._text;
            expect(yderName).to.eql(dataset.yderName);
        }       // Address
        if (dataset.customAddress == 'Ja') {
            let address = pdcComponent.healthProvider.organization.addr;

            expect(address.streetAddressLine._text).to.eql(
                dataset.streetAddressLine
            );
            expect(address.postalCode._text).to.eql(dataset.postalCode);
            expect(address.city._text).to.eql(dataset.city);
            expect(address.country._text).to.eql(dataset.country);
        }

        // Work Phone
        let workPhone =
            pdcComponent.healthProvider.organization.telecom.__value;

        if (dataset.phoneWP == '' || dataset.phoneWP == null) {
            expect(workPhone).to.eql('tel:null');
        } else {
            expect(workPhone).to.eql('tel:' + dataset.phoneWP);
        }
    }
);
//#endregion
//#endregion

//#region Temporary Address Fixtures
// This fixture expects data table to equal values in temporary address pdc component
Then(
    'midlertidig adresse med {string} har datasæt {string} i FSK stamkortet',
    async function (TempAdressId, Datasaet) {
        // Getting data table
        const dataset = this.datasets.get(Datasaet);

        // Getting pdc component from last document
        const pdcComponent = this.getLastDocument();

        // Expecting values in data table to equal data in pdc component
        // Temporary Address Id
        TempAdressId = this.replaceVariables(TempAdressId);
        let ResponseTempAdressId = pdcComponent.observation.id.__extension;

        expect(ResponseTempAdressId).to.eql(TempAdressId);

        // Street Address Line
        let streetAddress =
            pdcComponent.observation.value.streetAddressLine._text;

        expect(streetAddress).to.eql(dataset.streetAddressLine);

        // Postal Code
        let postalCode = pdcComponent.observation.value.postalCode._text;

        expect(postalCode).to.eql(dataset.postalCode);

        // City
        let city = pdcComponent.observation.value.city._text;

        expect(city).to.eql(dataset.city);

        // Country
        let country = pdcComponent.observation.value.country._text;

        expect(country).to.eql(dataset.country);

        // Useable Period 1
        if (pdcComponent.observation.effectiveTime.high.__value != undefined) {
            let useablePeriod1 =
                pdcComponent.observation.effectiveTime.low.__value.substring(
                    0,
                    8
                );

            expect(useablePeriod1).to.eql(dataset.useablePeriod1);
        }

        // Useable Period 2
        if (pdcComponent.observation.effectiveTime.high.__value != undefined) {
            let useablePeriod2 =
                pdcComponent.observation.effectiveTime.high.__value.substring(
                    0,
                    8
                );

            expect(useablePeriod2).to.eql(dataset.useablePeriod2);
        }
    }
);

// This fixture expects data table to equal values in temporary address pdc component (SKR)
Then(
    'midlertidig adresse med {string} har datasæt {string} i SKR stamkortet',
    async function (TempAdressId, Datasaet) {
        // Getting data table
        const dataset = this.datasets.get(Datasaet);

        // Getting pdc component from last document
        const pdcComponent = this.getLastDocument();

        // Expecting values in data table to equal data in pdc component
        // Temporary Address Id
        TempAdressId = this.replaceVariables(TempAdressId);
        let ResponseTempAdressId = pdcComponent.temporaryAddress.id.__extension;

        expect(ResponseTempAdressId).to.eql(TempAdressId);

        // Street Address Line
        let streetAddress =
            pdcComponent.temporaryAddress.addr.streetAddressLine._text;

        expect(streetAddress).to.eql(dataset.streetAddressLine);

        // Postal Code
        let postalCode = pdcComponent.temporaryAddress.addr.postalCode._text;

        expect(postalCode).to.eql(dataset.postalCode);

        // City
        let city = pdcComponent.temporaryAddress.addr.city._text;

        expect(city).to.eql(dataset.city);

        // Country
        let country = pdcComponent.temporaryAddress.addr.country._text;

        expect(country).to.eql(dataset.country);

        // Useable Period
        if (pdcComponent.temporaryAddress.addr.useablePeriod.length > 0) {
            let useablePeriod1 =
                pdcComponent.temporaryAddress.addr.useablePeriod[0].__value;

            expect(useablePeriod1).to.eql(dataset.useablePeriod1);

            let useablePeriod2 =
                pdcComponent.temporaryAddress.addr.useablePeriod[1].__value;

            expect(useablePeriod2).to.eql(dataset.useablePeriod2);
        } else if (dataset.useablePeriod2 != '') {
            let useablePeriod2 =
                pdcComponent.temporaryAddress.addr.useablePeriod.__value;

            expect(useablePeriod2).to.eql(dataset.useablePeriod2);
        } else if (dataset.useablePeriod1 != '') {
            let useablePeriod1 =
                pdcComponent.temporaryAddress.addr.useablePeriod.__value;

            expect(useablePeriod1).to.eql(dataset.useablePeriod1);
        }
    }
);

// This fixture makes sure that temporary address exists or not exists based on the given status
Given(
    'midlertidig adresse {string} på {string} stamkort',
    { timeout: -1 },
    async function (Status, PatientCPR) {
        // Replacing CPR
        PatientCPR = this.replaceVariables(PatientCPR);

        // Getting pdc documents/entries
        let entry_sfskRepositry = await getPDCEntries(this.attach, PatientCPR, 3);

        // Finding temporary address pdc component
        const pdcComponent = await jq.run(
            `.[]|select(.observation.code.__code=="TempAddrTypedIn")`,
            entry_sfskRepositry,
            { input: 'json', output: 'json' }
        );

        // Switch case based on status
        switch (Status) {
            // Case temporary address exists
            case 'findes':
                // If temporary address pdc compont doesn't exist
                if (pdcComponent == '') {
                    // Generating temporary address
                    let TempAdressId = v4();

                    // Creating temporary address
                    await CreateTemporaryAddress(
                        this.attach,
                        TempAdressId,
                        PatientCPR
                    );
                }
                break;
            // Case temporary address does not exist
            case 'findes ikke':
                // If language pdc component exist
                if (pdcComponent != '') {
                    // Finding temporary address id
                    TempAdressId = pdcComponent.observation.id.__extension;

                    // Deleting temporary address
                    await DeleteTemporaryAddress(
                        this.attach,
                        TempAdressId,
                        PatientCPR
                    );
                }
                break;
            default:
                // Throwing error if status is unknown
                throw new Error(`Unknown status "${Status}"`);
        }
        // Pushing pdc component
        this.documents.push(pdcComponent);
    }
);
//#endregion

//#region Address Protection
// This fixtures expects the patient to be address protected
Given('{string} har adressebeskyttelse',
    { timeout: -1 },
    async function (PatientCPR) {
        // Replacing CPR
        PatientCPR = this.replaceVariables(PatientCPR);

        // Getting patient details
        let dtg = await getPatientDetails(this.attach, PatientCPR);

        // Finding address protection
        const addressProtection = await jq.run(
            `.[]|select(.description=="Personen fik privatmarkeret sin adresse")`,
            dtg.body.events,
            { input: 'json', output: 'json' }

        );

        // Expecting patient to be address protected
        expect(addressProtection.description).to.eql(
            'Personen fik privatmarkeret sin adresse'
        );
    });

// This fixture
Then('CDA indeholder borgeroplysninger', async function (dataTable) {
    // Getting document entries
    if (this.getLastResponse().body.response.attachments[0] != null) {
        entries = this.getLastResponse().body.response.attachments[0].ClinicalDocument.component
            .structuredBody.component.section.entry;
    } else {
        entries = this.getLastResponse().body.response.main.Envelope.Body.RetrieveDocumentSetResponse.
            DocumentResponse.Document._data.ClinicalDocument.component.structuredBody.component.section.entry;
    }

    // Parsing data table
    const data = this.parseDataTable(dataTable);

    // Finding entry/document that contains patient name (PN)
    const entry = await jq.run(
        `.[]|select(.observation.value[]|select(."__{http://www.w3.org/2001/XMLSchema-instance}type"?=="PN"))`,
        entries,
        { input: 'json', output: 'json' }
    );

    // Finding name in entry/document
    const name = await jq.run(
        `.observation.value[]|select(."__{http://www.w3.org/2001/XMLSchema-instance}type"?=="PN")`,
        entry,
        { input: 'json', output: 'json' }
    );

    // Expecting first and last name in entry/document to equal values in data table
    expect(name.given._text).to.eql(data.fornavn);
    expect(name.family._text).to.eql(data.efternavn);

    // Finding address in entry/document
    const address = await jq.run(
        `.observation.value[]|select(."__{http://www.w3.org/2001/XMLSchema-instance}type"?=="AD")`,
        entry,
        { input: 'json', output: 'json' }
    );

    // Expecting address information in entry/document to equal values in data table
    expect(address.city._text).to.eql(data.by);
    expect(address.postalCode._text).to.eql(data.postkode);
    expect(address.streetAddressLine._text).to.eql(data.vej);
});
//#endregion

Then(
    'svaret fra {string} indeholder en advarsel om {string}',
    async function (ResponseName, Error) {
        let r = this.variables.get(ResponseName);

        let response =
            this.variables.get(ResponseName).body.response.main.Envelope.Body
                .AdhocQueryResponse.RegistryErrorList.RegistryError;
        if (response.length > 0) {
            const error = await jq.run(
                `.[] | select(.__codeContext == "urn:dk:nsi:${Error}")`,
                response,
                { input: 'json', output: 'json' }
            );
            expect(error.__codeContext).to.eql('urn:dk:nsi:' + Error);
        } else {
            expect(response.__codeContext).to.eql('urn:dk:nsi:' + Error);
        }
    }
);

Then('retursvar med {string} indeholder {string} lig med {string}', function (id, variable, value) {
    const pdcComponent = this.getLastDocument();
    id = this.replaceVariables(id);
    value = this.replaceVariables(value);

    let ResponseId = pdcComponent.observation.id.__extension;
    expect(id).to.eql(ResponseId);

    const variableMap = {
        '__codeSystem': 'observation.code.__codeSystem',
        '__codeSystemName': 'observation.code.__codeSystemName',
        '__displayName': 'observation.code.__displayName',
        '__root': 'observation.id.__root',
        '__extension': 'observation.templateId.__extension',
        'templateId.__root': 'observation.templateId.__root',
        'author.templateId.__root': 'observation.author.templateId.__root',
        'idextension': 'observation.id.__extension',
        'value.__AssigningAutorityName': 'observation.value[0].__assigningAuthorityName',
        'value.__extension': 'observation.value[0].__extension',
        'value.__root': 'observation.value[0].__root',
        'assignedAuthor': 'observation.author.assignedAuthor.code.__code',
        'assignedPersonGiven': 'observation.author.assignedAuthor.assignedPerson.name.given._text',
        'assignedPersonFamily': 'observation.author.assignedAuthor.assignedPerson.name.family._text',
        'authorDisplayName': 'observation.author.assignedAuthor.code.__displayName'
    };
    const path = variableMap[variable];
    if (path == undefined) {
        throw new Error(`Unknown element "${variable}"`);
    }

    const getValue = (obj, path) => {
        return path.split('.').reduce((pdc, element) => pdc && pdc[element], obj);
    };

    const actualValue = getValue(pdcComponent, path);
    expect(actualValue).to.eql(value);
});

Then('FSKretursvar indeholder {string} lig med {string}', function (variable, value) {
    let response =
        this.getLastResponse().body.response.main.Envelope.Body.RetrieveDocumentSetResponse
            .DocumentResponse.Document._data.ClinicalDocument
    console.log(response)
    switch (variable) {
        case '__codeSystem':
            expect(response.code.__codeSystem).to.eql(value);
            break;
        case '__codeSystemName':
            expect(response.code.__codeSystemName).to.eql(value);
            break;
        case '__displayName':
            expect(response.code.__displayName).to.eql(value);
            break;
        case '__root':
            expect(response.id.__root).to.eql(value);
            break;
        case '__extension':
            expect(response.templateId.__extension).to.eql(value);
            break;
        case 'templateId.__root':
            expect(response.templateId.__root).to.eql(value);
            break;
    }
});

//#region Contact Information
//This fixture is used to test a validation message, where the cpr numbers are different in the body
When(
    'Opdater kontaktinformation kaldes gennem DRG med {string} og {string}',
    async function (cpr, invalidCpr) {
        const res = await UpdateContactInformationInvalidCpr(
            this.attach,
            cpr,
            invalidCpr
        );
        this.responses.push(res);
    }
);

Then('kontaktoplysninger med {string} har telefonnummer {string} i FSK stamkortet',
    async function (patientContactId, number) {
        const pdcComponent = this.getLastDocument();
        patientContactId = this.replaceVariables(patientContactId);
        number = this.replaceVariables(number);

        ResponsepatientContactId = pdcComponent.observation.id.__extension;

        expect(patientContactId).to.eql(ResponsepatientContactId);
        expect(pdcComponent.observation.value.__value).to.eql('tel:' + number);
    }
);

When(
    'hent stamkort kaldes gennem DRG med {string} og {string}',
    async function (cpr, invalidCpr) {
        cpr = this.replaceVariables(cpr);
        invalidCpr = this.replaceVariables(invalidCpr);
        const res = await GetPersonalDataCardInvalidCpr(
            this.attach,
            cpr,
            invalidCpr
        );

        this.responses.push(res);
    }
);
//#endregion
