const {
    Before,
    BeforeAll,
    After,
    Given,
    Then,
    When,
} = require('@cucumber/cucumber');
const { createBlurring, GetBlurredOrganisations, GetActiveOrgBlurrings, GetActiveOrgBlurringsforCVR, createOrgBlurring } = require('dataaccess/idsas');
const jq = require('node-jq');
const { expect, delay, parseRelativeDate } = require('../util');
const roles = new Map(require('../../config/roles.json'));


///////////////////////////////////////////////////////////////
//              Given statements                             //
///////////////////////////////////////////////////////////////

//følgende skal oprette en sløring på borger
Given('sløring er oprettet for {string} med {string} på dato {string}',
    { timeout: -1 },
    async function (patientId, CVR, endDateTime) {
        patientId = this.replaceVariables(patientId);
        endDateTime = this.replaceVariables(endDateTime);
        CVR = this.replaceVariables(CVR);
        date = parseRelativeDate(endDateTime);

        response = await createBlurring(this.attach, patientId, date, CVR);
    });

Given('sløring er oprettet af for {string} på dato {string} og svar gemmes som {string}',
    async function (patientId, endDateTime, resp) {
        patientId = this.replaceVariables(patientId);
        endDateTime = this.replaceVariables(endDateTime);

        const response = await createBlurring(this.attach, patientId, endDateTime, CVR);
        this.variables.set(resp, response);
        this.responses.push(response);
    });

//denne gør at vi venter, således at sløring kan nå at blive oprettet
Given('vi afventer {int} millisekunder',
    { timeout: -1 }, async function (ms) {
        if (ms < 0) {
            throw new Error('Delay must be bigger than 0');
        }
        await delay(ms);
    });

Given('der ses slørede CVR for borger {string} og gemmes som {string}', async function (patientId, resp) {
    patientId = this.replaceVariables(patientId);
    const response = await GetBlurredOrganisations(this.attach, patientId);
    this.variables.set(resp, response);
    this.responses.push(response);
});

Given('der eksisterer ikke en sløring for borger {string} for {string}',
    async function (patientId, CVR) {
        patientId = this.replaceVariables(patientId);
        CVR = this.replaceVariables(CVR);
        let response = await GetBlurredOrganisations(this.attach, patientId);

        if (Array.isArray(response.body.response.main.Envelope.Body.BlurredOrganisationsResponse.organisationId)) {
            const res = await jq.run(
                `.[]|select(.id._text=="${CVR}")`,
                response.body.response.main.Envelope.Body.BlurredOrganisationsResponse.organisationId,
                { input: 'json', output: 'json' }
            );
            //Hvis der findes en sløring for dette CVR, slettes denne
            if (res.id._text == CVR) {
                await createBlurring(this.attach, patientId, '2021-01-01T00:00:00.000Z', CVR);
                console.log('Sløring slettet');
                response = await GetBlurredOrganisations(this.attach, patientId);
            }
        } else {
            if (response.body.response.main.Envelope.Body.BlurredOrganisationsResponse.organisationId != undefined) {
                //Hvis der findes en sløring for dette CVR, slettes denne
                if (response.body.response.main.Envelope.Body.BlurredOrganisationsResponse.organisationId.id._text == CVR) {
                    await createBlurring(this.attach, patientId, '2021-01-01T00:00:00.000Z', CVR);
                    console.log('Sløring slettet');
                    response = await GetBlurredOrganisations(this.attach, patientId);
                }
            }
        }
    }
);

Given(
    'sløring for {string} {string} med {string} på dato {string}',
    async function (patientId, status, CVR, endDateTime) {
        patientId = this.replaceVariables(patientId)
        CVR = this.replaceVariables(CVR)
        endDateTime = this.replaceVariables(endDateTime)

        switch (status) {
            case "findes":
                await createBlurring(this.attach, patientId, endDateTime, CVR)
                break;
            case "findes ikke":
                await createBlurring(this.attach, patientId, "2024-01-01T00:00:00.000Z", CVR)
                break;
            default:
                break;
        }
    });

Given('datasæt {string} {string} i {string}', async function (data, status, responseName) {
    const dataset = this.datasets.get(data);
    let response = this.variables.get(responseName).body.response.main.Envelope.Body.ListAllActiveOrgBlurringsResponse.org;
    switch (status) {
        case "findes":
            if (Array.isArray(response)) {
                const res = await jq.run(
                    `.[]|select(.id._text=="${dataset.orgId}" and (.classification._text=="${dataset.classification}"))`,
                    response,
                    { input: 'json', output: 'json' }
                );
                expect(res.id._text).to.eql(dataset.orgId);
                expect(res.classification._text).to.eql(dataset.classification);
            } else {
                expect(response.id._text).to.eql(dataset.orgId);
                expect(response.classification._text).to.eql(dataset.classification);
            }
            break;
        case "findes ikke":
            if (Array.isArray(response)) {
                const res = await jq.run(
                    `.[]|select(.id._text=="${dataset.orgId}" and (.classification._text=="${dataset.classification}"))`,
                    response,
                    { input: 'json', output: 'json' }
                );
                expect(res).to.empty;
            } else {
                expect(response).to.equal("")
            }
            break;
        default:
            break;
    }
});

Given('datasæt {string} {string} i listen for specifikt CVR {string}', async function (data, status, responseName) {
    const dataset = this.datasets.get(data);
    let response = this.variables.get(responseName).body.response.main.Envelope.Body.ListOrgBlurringsForCVRResponse.org;
    switch (status) {
        case "findes":
            if (Array.isArray(response)) {
                const res = await jq.run(
                    `.[]|select(.id._text=="${dataset.orgId}" and (.classification._text=="${dataset.classification}"))`,
                    response,
                    { input: 'json', output: 'json' }
                );
                expect(res.id._text).to.eql(dataset.orgId);
                expect(res.classification._text).to.eql(dataset.classification);
            } else {
                expect(response.id._text).to.eql(dataset.orgId);
                expect(response.classification._text).to.eql(dataset.classification);
            }
            break;
        case "findes ikke":
            if (Array.isArray(response)) {
                const res = await jq.run(
                    `.[]|select(.id._text=="${dataset.orgId}" and (.classification._text=="${dataset.classification}"))`,
                    response,
                    { input: 'json', output: 'json' }
                );
                expect(res).to.empty;
            } else {
                expect(response).to.equal("")
            }
            break;
        default:
            break;
    }
});

Given('der findes en afdelingssløring på {string} {string}',
    { timeout: -1 },
    async function (classification, id) {
        classification = this.replaceVariables(classification);
        id = this.replaceVariables(id);

        let response = await GetActiveOrgBlurrings(this.attach);
        response = response.body.response.main.Envelope.Body.ListAllActiveOrgBlurringsResponse.org

        let organisationId
        if (Array.isArray(response)) {
            organisationId = await jq.run(
                `.[]|select(.id._text=="${id}" and (.classification._text=="${classification}"))`,
                response,
                { input: 'json', output: 'json' }
            );
        } else {
            organisationId = response.id._text;
        }

        if (organisationId.id == null) {
            await createOrgBlurring(this.attach, this.role, id, classification);
        }
    });

Given('der findes en sløring for {string} på {string}', async function (cpr, cvr) {
    cpr = this.replaceVariables(cpr);
    cvr = this.replaceVariables(cvr);

    response = await GetBlurredOrganisations(this.attach, cpr)
    let organisationId = response.body.response.main.Envelope.Body.BlurredOrganisationsResponse.organisationId

    if (Array.isArray(organisationId)) {
        organisationId = await jq.run(
            `.[]|select(.id._text=="${cvr}")`,
            organisationId,
            { input: 'json', output: 'json' }
        );
    }
    expect(organisationId.id._text).to.eql(cvr)
});

///////////////////////////////////////////////////////////////
//              Then statements                             //
///////////////////////////////////////////////////////////////

// tjekker om der er en aktiv sløring for et bestemt CVRnr
Then('{string} {string} i {string}', async function (cvr, status, responseName) {
    let response = this.variables.get(responseName);
    cvr = this.replaceVariables(cvr)
    let cvrSvar = cvr
    OrganisationID =
        response.body.response.main.Envelope.Body.BlurredOrganisationsResponse.organisationId;
    switch (status) {
        case 'findes':
            if (OrganisationID != undefined) {
                if (OrganisationID.length > 1) {
                    console.log("Flere organisationer");
                    OrganisationID.forEach(ID => {
                        console.log("Fundet CVR:" + ID.id._text);
                        if (ID.id._text === cvr) {
                            cvrSvar === ID.id._text;
                            expect(cvrSvar).to.eql(cvr);
                        }
                    })
                }
                else if (OrganisationID.length = 1) {
                    cvrSvar = OrganisationID.id._text;
                    expect(cvrSvar).to.eql(cvr);
                } else {
                    cvrSvar = "";
                    expect(cvrSvar).to.eql(cvr);
                    console.log("ingen slørede CVR");
                }
            } else {
                cvrSvar = "";
                expect(cvrSvar).to.eql(cvr);
                console.log("ingen slørede CVR");
            }
            console.log(`cvrSvar i 'findes': ${cvrSvar}`);
            console.log(`cvr i 'findes': ${cvr}`);
            break;
        case 'findes ikke':
            if (OrganisationID != undefined) {
                if (OrganisationID.length > 1) {
                    console.log("findes en eller flere organisationer");
                    OrganisationID.forEach(ID => {
                        console.log("Fundet CVR: " + ID.id._text);
                        cvrSvar === ID.id._text;
                        expect(cvrSvar).not.to.eql(cvr);
                    })
                } else if (OrganisationID.length = 1) {
                    cvrSvar = OrganisationID.id._text;
                    expect(cvrSvar).not.to.eql(cvr);
                } else {
                    cvrSvar = "";
                    expect(cvrSvar).not.to.eql(cvr);
                    console.log("ingen slørede CVR");
                }
            } else {
                cvrSvar = "";
                expect(cvrSvar).not.to.eql(cvr);
                console.log("ingen slørede CVR");
            }
            console.log(`cvrSvar i 'findes ikke': ${cvrSvar}`);
            console.log(`cvr i 'findes ikke': ${cvr}`);
            break;
        default:
            break;
    }
});



//dette tjekker resultatet i en oprettet sløring er af status OK
Then('der {string} en sløring for borger i {string}',
    async function (Status, ResponseName) {


        let response = this.variables.get(ResponseName);

        let sløring =
            response.body.response.main.Envelope.Body.BlurResponse._text

        console.log(JSON.stringify(response.body.response.main.Envelope.Body.BlurResponse._text, null, 2));


        switch (Status) {
            case 'findes':
                expect(sløring).to.eql('OK');
                break;
            case 'findes ikke':
                expect(sløring).to.eql('');
                break;
            default:
                break;
        }
    }
);

Then(
    'der oprettes en sløring for borger {string} for {string} med slutdato {string}',
    async function (patientCPR, CVR, endDateTime) {
        patientCPR = this.replaceVariables(patientCPR);
        date = parseRelativeDate(endDateTime);
        CVR = this.replaceVariables(CVR);

        response = await createBlurring(this.attach, patientCPR, date, CVR);
    }
);

Then('der findes en afdelingsløring i listen over slørede afdelinger og svar gemmes som {string}',
    async function (resp) {
        const response = await GetActiveOrgBlurrings(this.attach);
        this.variables.set(resp, response);
        this.responses.push(response);
    });

Then('der findes en afdelingsløring i listen over slørede afdelinger for specifikt CVR og svar gemmes som {string}',
    async function (resp) {
        const response = await GetActiveOrgBlurringsforCVR(this.attach);
        this.variables.set(resp, response);
        this.responses.push(response);
    });