const chai = require('chai');
const cbor = require('cbor');
const base45 = require('base45-js');
const pako = require('pako');
chai.use(require('chai-datetime'));
const expect = require('chai').expect;
const v4 = require('uuid').v4;

function findPathsAndValues(rootNode) {
    // result object with paths as key and the respective value.
    // contains only paths to leaf-nodes
    const results = {};
    // visit object nodes recursively
    function visitNode(node, ancestorName, isParentArray) {
        if (node.__value) {
            // if leaf - aka has _text
            results[ancestorName + '.__value'] = node.__value;
        }
        if (node._text) {
            // if leaf - aka has _text
            results[ancestorName + '._text'] = node._text;
        }
        if (node.__id) {
            // if leaf - aka has _text
            results[ancestorName + '.__id'] = node._text;
        }
        // for each key in Object, record the name and visit each object-node
        for (let key of Object.keys(node)) {
            const value = node[key];
            if (typeof value === 'object') {
                let name;
                if (isParentArray) {
                    name = ancestorName + '[' + key + ']';
                } else {
                    name = ancestorName + '.' + key;
                }
                visitNode(value, name, Array.isArray(value));
            }
        }
    }
    // start at rootNodcorrect data check

    //console.log(results);
    return results;
}

function parseAmount(str) {
    const amount = parseInt(str, 10);
    if (isNaN(amount)) {
        throw new Error("Unable to parseInt from '" + str + "'");
    }
    return amount;
}

function parseRelativeDate(relative, _now = new Date()) {
    const date = new Date(_now.getTime());

    if (typeof relative !== 'string') {
        return relative;
    }

    let sign;
    if (relative.match(/^now[+-]?/i)) {
        relative = relative.slice(3);
        sign = relative[0];
        relative = relative.slice(1);
    } else {
        return relative;
    }

    while (relative.length > 0) {
        const amount = parseAmount(relative);
        relative = relative.slice(`${amount}`.length);

        if (sign == '+') {
            switch (relative[0]) {
                case 'Y':
                    date.setFullYear(date.getFullYear() + amount);
                    break;
                case 'M':
                    date.setMonth(date.getMonth() + amount);
                    break;
                case 'D':
                    date.setDate(date.getDate() + amount);
                    break;
                case 'h':
                    date.setHours(date.getHours() + amount);
                    break;
                case 'm':
                    date.setMinutes(date.getMinutes() + amount);
                    break;
                case 's':
                    date.setSeconds(date.getSeconds() + amount);
                    break;
                default:
                    throw new Error("No match for '" + relative[0] + "'");
            }
        } else if (sign == '-') {
            switch (relative[0]) {
                case 'Y':
                    date.setFullYear(date.getFullYear() - amount);
                    break;
                case 'M':
                    date.setMonth(date.getMonth() - amount);
                    break;
                case 'D':
                    date.setDate(date.getDate() - amount);
                    break;
                case 'h':
                    date.setHours(date.getHours() - amount);
                    break;
                case 'm':
                    date.setMinutes(date.getMinutes() - amount);
                    break;
                case 's':
                    date.setSeconds(date.getSeconds() - amount);
                    break;
                default:
                    throw new Error("No match for '" + relative[0] + "'");
            }
        } else {
            throw new Error('Operator not recognized ' + sign);
        }

        relative = relative.slice(1);
    }
    return date.toISOString().replace(/\.\d+Z/, 'Z'); //Fjerner millisek da nogle services herunder BRS ikke kan lide dem;
}

// usage: await delay(2000);
function delay(ms) {
    return new Promise((resolve) => setTimeout(resolve, ms));
}

async function decodeJWT(token) {
    // remove leasing classification DK1: etc.
    const colonIdx = token.indexOf(':');
    token = token.slice(colonIdx + 1);

    token = base45.decode(token);

    // zlib inflate
    token = pako.inflate(new Uint8Array(token));

    const result = cbor.decodeFirstSync(token);
    [headers1, headers2, cbor_data, signature] = result.value;

    const greencarddata = cbor.decodeFirstSync(cbor_data);
    if (greencarddata.has(7)) {
        greencarddata.set(7, greencarddata.get(7).toString('utf-8'));
    }
    const obj = Object.fromEntries(greencarddata);
    if (obj['-260']) {
        obj['-260'] = Object.fromEntries(obj['-260']);
    }
    return obj;
}

function oid() {
    return Math.floor(
        Math.random() * (999999999 - 10000000) + 100000000
    ).toString();
}

module.exports = {
    decodeJWT,
    expect,
    findPathsAndValues,
    parseRelativeDate,
    delay,
    oid,
};
