#!groovy

node {
  def testsFailed = false
  try {
    stage('Checkout') {
      checkout scm
    }

    stage('Export feature files') {
      step([$class: 'XrayExportBuilder',
        serverInstance: '89d8e9d1-8aff-4f28-8830-5d1217f0d789',
        filter: '11608',
        filePath: 'cucumber/features',
      ])
    }

    stage('Test') {
      withCredentials([[$class: 'UsernamePasswordMultiBinding',
                    credentialsId: "${credentials}",
                    usernameVariable: 'CRED_USER',
                    passwordVariable: 'CRED_PASSWORD']]) {
        docker.image("registry.nspop.dk/tools/node-chrome:18").inside("--entrypoint=''"){
          dir("cucumber") {
            sh "npm install"

            try {
              sh "mkdir -p target"
              sh "DRG_USER=\"${CRED_USER}\" DRG_PASSWORD=\"${CRED_PASSWORD}\" ENV=\"${ENVIRONMENT}\" npm run test"
            } catch (err) {
              // dont yell - we upload test report
              testsFailed = true
            }
          }
        }
      }
    }

  } catch (err) {
    slackSend channel: 'arosii_ci', color: 'bad', message: "${env.JOB_NAME} ${env.BUILD_NUMBER} - Tests failed ... (<${env.BUILD_URL}|Open>)", tokenCredentialId: 'Slack-Token'
    throw err
  } finally {

    stage('Archive') {
      cucumber( 
        reportTitle: 'MS_NRT',
        fileIncludePattern: 'cucumber/target/*.json',
        trendsLimit: 10,
      )

      if (testsFailed) {
        if (fileExists("cucumber/target/report.json")) {
          String reportJson = readFile("cucumber/target/report.json")
          String[] lines = reportJson.split("\n")

          def fails = lines.findAll{ line-> line =~ "\"status\": \"failed\""}.size()

          slackSend channel: 'arosii_ci', color: 'bad', message: "${env.JOB_NAME} ${env.BUILD_NUMBER} A total of ${fails} tests failed ... (<${env.BUILD_URL}|Open>)", tokenCredentialId: 'Slack-Token'
        } else {
          slackSend channel: 'arosii_ci', color: 'bad', message: "${env.JOB_NAME} ${env.BUILD_NUMBER} - Tests failed ... (<${env.BUILD_URL}|Open>)", tokenCredentialId: 'Slack-Token'
          println("no file found")
        }
      }

      if (fileExists("cucumber/target/report.json")) {
        step([$class: 'XrayImportBuilder',
          serverInstance: '89d8e9d1-8aff-4f28-8830-5d1217f0d789',
          projectKey: 'NRT',
          endpointName: '/cucumber',
          importFilePath: 'cucumber/target/report.json',
          importToSameExecution: 'true'
        ])
      }
    }
    stage ('Clean') {
      deleteDir()
    }
  }
}

